#include <iostream>
#include  "buffer.h"
#include  <cctype>
#include  <cstring>
#include  <cstdlib>
#include  <stdexcept>

namespace atlas
{ namespace interpreter
  {

BufferedInput::BufferedInput (std::istream& s)
:
base_stream(s),line_buffer(),p(NULL),prompt(""),prompt2(""),def_ext(NULL),
temp_prompt(""),
readline(NULL),add_hist(NULL),line_no(1),cur_lines(0),input_stack(),
stream(&base_stream)
{}

BufferedInput::BufferedInput
(const char* pr, rl_type rl, add_hist_type ah,const char* pr2,const char* de)
:
base_stream(std::cin),line_buffer(),p(NULL),
prompt(pr),prompt2(pr2),def_ext(de),temp_prompt(""),
readline(rl),add_hist(ah),line_no(1),cur_lines(0),input_stack(),
stream(&base_stream)
{}

BufferedInput::~BufferedInput()
{ while (not input_stack.empty())
  { delete input_stack.top().stream;
    input_stack.pop();
  }
}

void BufferedInput::pop_file()
{ line_no = input_stack.top().line_no;
  delete input_stack.top().stream;
  input_stack.pop();
  stream= input_stack.empty() ? &base_stream : input_stack.top().stream;
}

void BufferedInput::close_includes()
{ while (not input_stack.empty())
  { delete input_stack.top().stream;
    input_stack.pop();
  }
  stream= &base_stream;
}

input_record::input_record
(const char* file_name, const char* def_ext,unsigned long line) :
 name(file_name), stream (new std::ifstream(file_name)), line_no(line)
{ try
  {
    if (def_ext!=NULL and not stream->is_open())
    { name += def_ext;
      stream->open(name.c_str());
    }
  }
  catch (...)
  { delete stream; throw; }
}

void BufferedInput::push_file(const char* name)
{ input_stack.push(input_record(name,def_ext,line_no+cur_lines));
  // reading will resume there
  if (input_stack.top().stream->good())
  { stream= input_stack.top().stream;
    line_no=1; cur_lines=0;
    // don't advance |line_no| when getting first line of new file
  }
  else
  { std::cerr << "failed to open input file '" << input_stack.top().name
              << "'." << std::endl;
    delete input_stack.top().stream;
    input_stack.pop();
// no need to call |pop_file|: |stream|, |line_no| and |cur_lines| are unchanged
  }
}

bool BufferedInput::getline()
{ std::string line; line_buffer="";
  line_no+=cur_lines; cur_lines=0;
  const char* pr=
               (temp_prompt.empty() ? prompt : (temp_prompt+" "+prompt2).c_str())
;
   while (stream->good())
  { 
    { do 
         if (stream==&std::cin) // which implies |input_stack.empty()|
         { prompt_length= std::strlen(pr);
           if (readline!=NULL)
           { char* l=readline(pr);
             if (l==NULL) // then |readline| failed, flag end of file
             { line=""; stream->setstate(std::ios_base::eofbit);
               std::cout << "^D\n";
             }
             else
             { line=l;
               if (add_hist!=NULL and *l!='\0') add_hist(l);
               else std::free(l);
             }
           }
           else { std::cout << pr; std::getline(std::cin,line,'\n'); }
         }
         else std::getline(*stream,line,'\n');
      while(line.empty() and not stream->good() and not input_stack.empty()
            and (pop_file(),true));
      if (line.empty() and not stream->good()) break;
      // no more input streams active, so give up
    }
    ++cur_lines;
    std::string::size_type l=line.length();
    while (l>0 and std::isspace(line[l-1])) --l;
    if (l<line.length()) line.erase(l); // remove trailing space
    if (l==0 or line[l-1]!='\\') { line_buffer+=line; break; }
    line.erase(l-1); line_buffer+=line; pr= "\\ "; // continuation prompt
  }
  line_buffer.push_back('\n'); // add newline that was suppressed on reading
  p=line_buffer.data();
return stream->good() or cur_lines>0 ;
  // delay reporting end of input if anything was read
}

void BufferedInput::locate (const char* p, int& line, int& column) const
{ line=line_no; column=p-line_buffer.data(); }

void BufferedInput::show_range
  (std::ostream& out,unsigned long l0, int c0, unsigned long l1, int c1)
  const
{ if (l1==line_no)
  { int pl=prompt_length; // offset of last line on the screen
    if (stream!=&std::cin or cur_lines>1)
    { if (not input_stack.empty())
        out << "In input file '" << input_stack.top().name << "', ";
      if (stream!=&std::cin) out << "line " << l0 << ":\n";
      out<<line_buffer; pl=0; // echo line in these cases
    }
    if (l0<l1) c0=0; // forget start column of multi-line range
    for (int i=pl+c0; i>0; --i) out<<' ';
    for (int i=c1; i>c0; --i) out<<'^';
    out << std::endl;
    if (l0<l1)
      if (l1-l0==1) out << "Range started in previous line\n";
      else out << "Range started " << (l1-l0) << "lines above\n";
  }
  else // range ended before the current line
  { out << "Range from line " << l0 << " column " << c0 
	<< " to line " << l1 << " column " << c1 << ".\n";
  }
}

void BufferedInput::push_prompt(char c) { temp_prompt.push_back(c); }
char BufferedInput::top_prompt() const
{ size_t l=temp_prompt.length();
  return l==0 ? '\0' : temp_prompt[l-1];
}
void BufferedInput::pop_prompt()
{ size_t l=temp_prompt.length();
  if (l>0) temp_prompt.erase(l-1);
}
void BufferedInput::reset()  { temp_prompt=""; }


BufferedInput* main_input_buffer=NULL;

  }
}

