
#ifndef BUFFER_H
#define BUFFER_H


#include<iostream>

#include <string>
#include <stack>
#include <fstream>



namespace atlas
{ namespace interpreter
 {

struct input_record
{ std::string name;
  std::ifstream* stream; // pointer owned by parent object
  unsigned long line_no;
  input_record(const char* file_name, const char* def_ext, unsigned long line);
};


class BufferedInput
{ typedef char* (*rl_type)(const char* );
     // |rl_type| is type of pointer to readline function
  typedef void (*add_hist_type) (const char* );
     // |add_hist_type| is type of pointer to |add_history| function
  
  std::istream& base_stream;
  std::string line_buffer; // current line
  const char* p; // buffer pointer
  const char *const prompt, *const prompt2; // primary and secondary prompt
  const char* def_ext; // default file extension
  std::string temp_prompt; // varying addition to secondary prompt
  const rl_type readline; // readline function
  const add_hist_type add_hist; // history function
  unsigned long line_no; // current line number
  int cur_lines,prompt_length; // local variables
  std::stack<input_record> input_stack; // active input streams
  std::istream* stream; 

  public:
    BufferedInput (std::istream& s);
        // associate line buffer to raw input stream
    BufferedInput (const char* prompt
                  , rl_type rl=NULL, add_hist_type=NULL 
		  , const char* prompt2="> "
                  , const char* def_ext=".rx");
        // use |stdin|, maybe with readline
    ~BufferedInput();

    char shift (); // inspect a new character
    void unshift (); // back up so last character will be reconsidered
    bool eol () const; // end of line: |true| if |shift| would return a newline
    bool getline ();
         // fetch a new line to replace current one; |true| if successful
    void push_file (const char* file_name);
    void close_includes();
    
    const char* point() const {return p;}
       // points at next char in line buffer
    void set_line_no (unsigned long l) { line_no=l; }
    void locate (const char* p, int& line, int& column) const;
    
    void show_range
     (std::ostream& out,unsigned long l0, int c0, unsigned long l1, int c1)
     const;
    
    void push_prompt(char c);
    char top_prompt() const;
    void pop_prompt();
    void reset();

private:
    void pop_file();
};


inline bool BufferedInput::eol() const { return p==0 or *p=='\0'; }

inline char BufferedInput::shift()
{ if (eol())
    if (not getline()) { p=NULL; return '\0'; } // signals file end
  return *p++;
}

inline void BufferedInput::unshift()
{ if (p!=NULL and p>line_buffer.data()) --p; }


extern BufferedInput* main_input_buffer;

 }
}
#endif

