#include  "built-in-types.h"
#include  <stdexcept>
#include  "lietype.h"
#include  "constants.h"
#include  <sstream>
#include  <cctype>
#include  "basic_io.h"
#include  "prerootdata.h"
#include  "dynkin.h"
#include  "arithmetic.h"
#include  "lattice.h"
#include  "latticetypes.h"
#include  "matrix.h"
#include  "smithnormal.h"
#include  "tags.h"
#include  "tori.h"
#include  "dynkin.h"
#include  "poset.h"
#include  "prettyprint.h"
#include  "ioutils.h"
#include  "realredgp_io.h"
#include  "blocks.h"
#include  "block_io.h"
#include  "kgb.h"
#include  "kgb_io.h"
#include  "kl.h"
#include  "klsupport.h"
#include  "kl_io.h"
#include  "wgraph.h"
#include  "wgraph_io.h"

namespace atlas { namespace interpreter {
namespace { 
            inline std::istream& skip_punctuation(std::istream &is)
            { char c; do is>>c; while (is and std::ispunct(c));
                return is.unget();
            }
            
            void Lie_type_wrapper(expression_base::level l)
               throw(std::bad_alloc,std::runtime_error)
            { std::istringstream is(get<string_value>()->val);
              Lie_type_ptr result(new Lie_type_value);
              size_t total_rank=0; char c;
              while (skip_punctuation(is)>>c) // i.e., until |not is.good()|
              { size_t rank;
                if (not (is>>rank))
                  throw std::runtime_error
                   ("Error in type string '"+is.str()+"' for Lie type");
            
                result->add_simple_factor(c,rank);
                  // this may |throw| a |runtime_error| as well
                if ((total_rank+=rank)>constants::RANK_MAX)
                  throw std::runtime_error
                  ("Total rank exceeds implementation limit "+str(constants::RANK_MAX));
            
              }
              if (l!=expression_base::no_value)
                push_value(result);
            }
            
            void Lie_type_coercion()
            { return Lie_type_wrapper(expression_base::single_value); }
            
            void Cartan_matrix_wrapper(expression_base::level l)
            { shared_Lie_type t=get<Lie_type_value>();
              if (l==expression_base::no_value)
                return;
              matrix_ptr result(new matrix_value(t->val.Cartan_matrix()));
              push_value(result);
            }
            
            void type_of_Cartan_matrix_wrapper (expression_base::level l)
            { shared_matrix m=get<matrix_value>();
              setutils::Permutation pi;
              lietype::LieType lt=dynkin::Lie_type(m->val,true,true,pi);
              if (l==expression_base::no_value)
                return;
              push_value(new Lie_type_value(lt));
              push_value(new vector_value(latticetypes::CoeffList(pi.begin(),pi.end())));
              if (l==expression_base::single_value)
                wrap_tuple(2);
            }
            
            void Lie_rank_wrapper(expression_base::level l)
            { shared_Lie_type t=get<Lie_type_value>();
              if (l!=expression_base::no_value)
                push_value(new int_value(t->val.rank()));
            }
            void semisimple_rank_wrapper(expression_base::level l)
            { shared_Lie_type t=get<Lie_type_value>();
              if (l!=expression_base::no_value)
                push_value(new int_value(t->val.semisimple_rank()));
            }
            
            void nr_factors_wrapper(expression_base::level l)
            { shared_Lie_type t=get<Lie_type_value>();
              if (l!=expression_base::no_value)
                push_value(new int_value(t->val.size()));
            }
            
            void Smith_Cartan_wrapper(expression_base::level l)
            { shared_Lie_type t=get<Lie_type_value>();
              if (l==expression_base::no_value)
                return;
              vector_ptr inv_factors (new vector_value(latticetypes::CoeffList()));
              latticetypes::WeightList b = t->val.Smith_basis(inv_factors->val);
              push_value(new matrix_value(latticetypes::LatticeMatrix(b)));
              push_value(inv_factors);
              if (l==expression_base::single_value)
                 wrap_tuple(2);
            }
            
            void filter_units_wrapper (expression_base::level l)
            { shared_vector inv_f=get_own<vector_value>();
              shared_matrix basis=get_own<matrix_value>();
              if (inv_f->val.size()!=basis->val.numColumns())
                throw std::runtime_error ("Size mismatch "+
            
                  str(inv_f->val.size())+':'+str(basis->val.numColumns()));
              if (l==expression_base::no_value)
                return;
            
              size_t i=0;
              while (i<inv_f->val.size())
                if (inv_f->val[i]!=1) ++i; // keep invariant factor and column
                else
                { inv_f->val.erase(inv_f->val.begin()+i);
                    basis->val.eraseColumn(i);
                }
              push_value(basis); push_value(inv_f);
              if (l==expression_base::single_value)
                  wrap_tuple(2);
            }
            
            latticetypes::LatticeMatrix 
            annihilator_modulo
            (const latticetypes::LatticeMatrix& M,
             latticetypes::LatticeCoeff denominator)
            
            { const size_t m=M.numRows(); // determines dimension of output
            
              latticetypes::WeightList b; matrix::initBasis(b,m);
                // standard basis of rank $m$
            
              latticetypes::CoeffList lambda; // invariant factors; in fact non-negative
              smithnormal::smithNormal(lambda,b.begin(),M);
                // find Smith basis
            
              latticetypes::LatticeMatrix A
                (latticetypes::LatticeMatrix(b).inverse().transposed());
            
              for (size_t j = 0; j < lambda.size(); ++j)
              { unsigned long c =
                  denominator/arithmetic::gcd(denominator,(unsigned long)(lambda[j]));
                for (size_t i=0; i<m; ++i) A(i,j)*=c; // multiply column |j| by |c|
              }
              return A;
            }
            
            void ann_mod_wrapper(expression_base::level l)
            { shared_int d=get<int_value>();
              shared_matrix m=get<matrix_value>();
            
              if (l!=expression_base::no_value)
                push_value(new matrix_value(annihilator_modulo(m->val,d->val)));
            }
            
            void replace_gen_wrapper (expression_base::level l)
            { shared_matrix new_generators=get<matrix_value>();
              push_tuple_components(); // a pair as returned by \.{Smith\_Cartan}
              shared_vector inv_f=get<vector_value>();
              shared_matrix generators=get_own<matrix_value>();
               // start with old generators
            
              if (new_generators->val.numRows()!=generators->val.numRows())
                throw std::runtime_error("Column lengths do not match");
            
              if (inv_f->val.size()!=generators->val.numColumns())
                throw std::runtime_error("Number of columns mismatch");
            
            
              size_t k=0; // index to replacement generators
              for (size_t j=0; j<inv_f->val.size(); ++j)
                if (inv_f->val[j]!=1)
                   // replace column |j| by column |k| from |new_generators|
                { if (k>=new_generators->val.numColumns())
                    throw std::runtime_error ("Not enough replacement columns");
            
                  generators->val.set_column(j,new_generators->val.column(k));
                  ++k;
                }
              if (k<new_generators->val.numColumns())
                    throw std::runtime_error ("Too many replacement columns");
            
              if (l!=expression_base::no_value)
                push_value(generators);
            }
            
            lietype::InnerClassType transform_inner_class_type
              (const char* s, const lietype::LieType& lt)
            throw (std::bad_alloc, std::runtime_error)
            { static const std::string types(atlas::lietype::innerClassLetters);
                // |"Ccesu"|
              lietype::InnerClassType result; // initially empty
              std::istringstream is(s);
              char c;
              size_t i=0; // position in simple factors of Lie type |lt|
              while (skip_punctuation(is)>>c)
                
                { if (types.find(c) == std::string::npos) throw std::runtime_error
                    (std::string("Unknown inner class symbol `")+c+"'");
                  if (i>= lt.size()) throw std::runtime_error("Too many inner class symbols");
                
                  lietype::TypeLetter t = lt[i].type(); size_t r=lt[i].rank();
                  if (c=='C') // complex inner class, which requires two equal simple factors
                  { if (i+1>=lt.size() or lt[i+1]!=lt[i]) throw std::runtime_error 
                      ("Complex inner class needs two identical consecutive types");
                
                    result.push_back('C');
                    i+=2; // advance past both simple factors
                  }
                  else if (c=='c' or c=='e')
                  { result.push_back('c'); ++i; } // synonyms
                  else if (c=='s')
                  { 
                    if (t=='A' and r>=2 or
                        t=='D' and r%2!=0 or
                        t=='E' and r==6 or
                        t=='T')
                      result.push_back('s');
                    else result.push_back('c');
                    ++i;
                  }
                  else if (c=='u') // unequal rank, the only remaining possibility
                  { 
                    { if (t=='D') result.push_back(r%2==0 ? 'u' : 's');
                      else if (t=='A' and r>=2 or t=='E' and r==6 or t=='T') result.push_back('s');
                      else throw std::runtime_error 
                        (std::string("Unequal rank class is meaningless for type ")+t+str(r));
                    
                    }
                    ++i;
                  }
                }
              if (i< lt.size()) throw std::runtime_error("Too few inner class symbols");
            
              return result;
            }
            
            void basic_involution_wrapper(expression_base::level l)
            { push_tuple_components();
            shared_string str=get<string_value>();
              shared_Lie_type t=get<Lie_type_value>();
            push_value(new matrix_value  (lietype::involution
                       (t->val,transform_inner_class_type(str->val.c_str(),t->val))));
            }
            
            void based_involution_wrapper(expression_base::level l)
            { shared_string s = get<string_value>();
            shared_matrix basis = get<matrix_value>();
            shared_Lie_type type = get<Lie_type_value>();
            
              size_t r=type->val.rank();
              if (basis->val.numRows()!=r or basis->val.numRows()!=r)
                throw std::runtime_error 
                ("Basis should be given by "+str(r)+'x'+str(r)+" matrix");
            
            
              latticetypes::LatticeMatrix inv=lietype::involution
                    (type->val,transform_inner_class_type(s->val.c_str(),type->val));
              try
              { push_value(new matrix_value(inv.on_basis(basis->val.columns()))); }
              catch (std::runtime_error&) // relabel |"Inexact integer division"|
              { throw std::runtime_error
                  ("Inner class is not compatible with given lattice");
            
              }
            }
            
            lietype::LieType type_of_datum(const rootdata::RootDatum& rd)
            { latticetypes::LatticeMatrix Cartan = rd.cartanMatrix();
            lietype::LieType t = dynkin::Lie_type(Cartan);
              if (!rd.isSemisimple())
                for (size_t i=rd.semisimpleRank(); i<rd.rank(); ++i)
                  t.push_back(lietype::SimpleLieType('T',1));
              return t;
            }
            
            void type_of_root_datum_wrapper(expression_base::level l)
            { shared_root_datum rd(get<root_datum_value>());
              push_value(new Lie_type_value(type_of_datum(rd->val)));
            }
            
            void root_datum_wrapper(expression_base::level l)
            { shared_matrix lattice=get<matrix_value>();
              shared_Lie_type type=get<Lie_type_value>();
              if (lattice->val.numRows()!=lattice->val.numColumns()  or
                  lattice->val.numRows()!=type->val.rank())
                throw std::runtime_error
                ("Sub-lattice matrix should have size " 
            
                  +str(type->val.rank())+'x'+str(type->val.rank()));
              try
              {
                prerootdata::PreRootDatum prd(type->val,lattice->val.columns());
                if (l!=expression_base::no_value)
                  push_value(new root_datum_value  (rootdata::RootDatum(prd)));
              }
              catch (std::runtime_error& e)
              { if (e.what()[0]=='I') // |"Inexact integer division"|
                  throw std::runtime_error
                    ("Sub-lattice does not contain the root lattice");
            
                throw; // |"Dependent lattice generators"|, no need to rephrase
            
              }
            }
            
            void raw_root_datum_wrapper(expression_base::level l)
            { size_t rank = get<int_value>()->val;
              shared_row simple_coroots=get<row_value>();
              shared_row simple_roots=get<row_value>();
            
              if (simple_roots->val.size()!=simple_coroots->val.size())
                throw std::runtime_error
                ("Numbers "+str(simple_roots->val.size())+","
                  +str(simple_coroots->val.size())+  " of simple (co)roots mismatch");
            
            
              latticetypes::WeightList s,c;
              s.reserve(simple_roots->val.size());
              c.reserve(simple_roots->val.size());
            
              for (size_t i=0; i<simple_roots->val.size(); ++i)
              { const latticetypes::Weight& sr
                  =force<vector_value>(simple_roots->val[i].get())->val;
                const latticetypes::Weight& scr
                  =force<vector_value>(simple_coroots->val[i].get())->val;
            
                if (sr.size()!=rank or scr.size()!=rank)
                throw std::runtime_error
                  ("Simple (co)roots not all of size "+str(rank));
                s.push_back(sr);
                c.push_back(scr);
              }
            
              prerootdata::PreRootDatum prd(s,c,rank);
              try { setutils::Permutation dummy;
                dynkin::Lie_type(prd.Cartan_matrix(),true,true,dummy);
              }
              catch (std::runtime_error& e)
              { throw std::runtime_error("Invalid simple (co)roots"); }
              if (l!=expression_base::no_value)
                push_value(new root_datum_value  (rootdata::RootDatum(prd)));
            }
            
            void quotient_basis_wrapper(expression_base::level l)
            { shared_row L=get<row_value>();
              // and leave Lie type on stack for $Smith\_Cartan$
              Smith_Cartan_wrapper(expression_base::multi_value);
              shared_value SC_basis = *(execution_stack.end()-2);
              shared_value invf = *(execution_stack.end()-1);
              wrap_tuple(2); // |replace_gen| wants as first argument a tuple
            push_value(SC_basis);
              push_value(invf);
              filter_units_wrapper(expression_base::multi_value);
              shared_vector v=get<vector_value>();
              // and leave |C| for call to $mm\_prod$
            
              latticetypes::LatticeMatrix M(v->val.size(),L->length());
              size_t d=1;
              
              { std::vector<unsigned long> denom(L->length());
                for (size_t j=0; j<L->length(); ++j)
                { const latticetypes::RatWeight& gen =
                    force<rational_vector_value>(&*L->val[j])->val;
                  denom[j] = gen.denominator();
                  d=arithmetic::lcm(d,denom[j]);
              
                  if (gen.size()!=v->val.size())
                    throw std::runtime_error
                      ("Length mismatch for generator "+str(j) +": "
              
                      +str(gen.size()) + ':' + str(v->val.size()));
              
                  M.set_column(j,gen.numerator());
                  for (size_t i=0; i<v->val.size(); ++i)
                    if (v->val[i]*M(i,j)%long(denom[j])!=0) // must use signed arithmetic!!
              	throw std::runtime_error("Improper generator entry: "
              
                       +str(M(i,j))+'/'+str(denom[j])+" not a multiple of 1/"
                       +str(v->val[i]));
                }
              
                for (size_t j=0; j<L->length(); ++j)
                  // convert modulo $\Z$ and to common denominator |d|
                { size_t f=d/denom[j];
                  for (size_t i=0; i<v->val.size(); ++i)
                    M(i,j) = arithmetic::remainder(M(i,j),denom[j])*f;
                }
              }
              push_value(new matrix_value(annihilator_modulo(M,d)));
            mm_prod_wrapper(expression_base::single_value);
            replace_gen_wrapper(l); // pass level parameter to final call
            }
            
            void quotient_datum_wrapper(expression_base::level l)
            { shared_value L = pop_value();
              shared_value lt= pop_value();
              push_value(lt); // the Lie type, for call of $root\_datum$
            push_value(lt);
              push_value(L); quotient_basis_wrapper(expression_base::single_value);
            root_datum_wrapper(l); // pass level parameter to final call
            }
            
            void simply_connected_datum_wrapper(expression_base::level l)
            { if (l==expression_base::no_value)
              { execution_stack.pop_back();
                return; // no possibilities of errors, so avoid useless work
              }
              size_t rank =
                force<Lie_type_value>(execution_stack.back().get())->val.rank();
              push_value(new int_value(rank));
              id_mat_wrapper(expression_base::single_value);
            root_datum_wrapper(expression_base::single_value);
            }
            
            void adjoint_datum_wrapper(expression_base::level l)
            { if (l==expression_base::no_value)
              { execution_stack.pop_back();
                return; // no possibilities of errors, so avoid useless work
              }
              push_value(execution_stack.back()); // duplicate Lie type argument
              Cartan_matrix_wrapper(expression_base::single_value);
              transpose_mat_wrapper(expression_base::single_value);
              shared_matrix M=get_own<matrix_value>();
              for (size_t i=0; i<M->val.numRows(); ++i)
                if (M->val(i,i)==0) M->val(i,i)=1;
              push_value(M);
            root_datum_wrapper(expression_base::single_value);
            }
            
            void SL_wrapper(expression_base::level l)
            { shared_int n(get<int_value>());
              if (n->val<1) throw std::runtime_error("Non positive argument for SL");
            
              if (l==expression_base::no_value)
                return;
              const size_t r=n->val-1;
              Lie_type_ptr type(new Lie_type_value());
              if (r>0) type->add_simple_factor('A',r);
              push_value(type);
              matrix_ptr lattice
                 (new matrix_value(latticetypes::LatticeMatrix()));
              matrix::identityMatrix(lattice->val,r);
              for (size_t i=0; i+1<r; ++i) // not |i<r-1|, since |r| unsigned and maybe 0
                lattice->val(i,i+1)=-1;
              push_value(lattice);
            root_datum_wrapper(expression_base::single_value);
            }
            
            void GL_wrapper(expression_base::level l)
            { shared_int n(get<int_value>());
              if (n->val<1) throw std::runtime_error("Non positive argument for GL");
              if (l==expression_base::no_value)
                return;
              const size_t r=n->val-1;
              Lie_type_ptr type(new Lie_type_value());
              if (r>0) type->add_simple_factor('A',r);
              type->add_simple_factor('T',1);
              push_value(type);
              matrix_ptr lattice
                 (new matrix_value(latticetypes::LatticeMatrix()));
              matrix::identityMatrix(lattice->val,r+1);
              for (size_t i=0; i<r; ++i)
              { lattice->val(n->val-1,i)=1; lattice->val(i,i+1)=-1; }
              push_value(lattice);
            root_datum_wrapper(expression_base::single_value);
            }
            
            void simple_roots_wrapper(expression_base::level l)
            { shared_root_datum rd(get<root_datum_value>());
              if (l==expression_base::no_value)
                return;
              latticetypes::WeightList srl
                (rd->val.beginSimpleRoot(),rd->val.endSimpleRoot());
              push_value(new matrix_value(atlas::latticetypes::LatticeMatrix(srl)));
            }
            
            void simple_coroots_wrapper(expression_base::level l)
            { shared_root_datum rd(get<root_datum_value>());
              if (l==expression_base::no_value)
                return;
              latticetypes::WeightList scl
                (rd->val.beginSimpleCoroot(),rd->val.endSimpleCoroot());
              push_value(new matrix_value(atlas::latticetypes::LatticeMatrix(scl)));
            }
            
            void datum_Cartan_wrapper(expression_base::level l)
            { shared_root_datum rd(get<root_datum_value>());
              if (l==expression_base::no_value)
                return;
              latticetypes::LatticeMatrix M = rd->val.cartanMatrix();
              push_value(new matrix_value(M));
            }
            
            void roots_wrapper(expression_base::level l)
            { shared_root_datum rd(get<root_datum_value>());
              if (l==expression_base::no_value)
                return;
              latticetypes::WeightList rl
                (rd->val.beginRoot(),rd->val.endRoot());
              push_value(new matrix_value(atlas::latticetypes::LatticeMatrix(rl)));
            }
            
            void coroots_wrapper(expression_base::level l)
            { shared_root_datum rd(get<root_datum_value>());
              if (l==expression_base::no_value)
                return;
              latticetypes::WeightList crl
                (rd->val.beginCoroot(),rd->val.endCoroot());
              push_value(new matrix_value(atlas::latticetypes::LatticeMatrix(crl)));
            }
            
            void root_coradical_wrapper(expression_base::level l)
            { shared_root_datum rd(get<root_datum_value>());
              if (l==expression_base::no_value)
                return;
              latticetypes::WeightList srl
                (rd->val.beginSimpleRoot(),rd->val.endSimpleRoot());
              srl.insert(srl.end(),rd->val.beginCoradical(),rd->val.endCoradical());
              push_value(new matrix_value(atlas::latticetypes::LatticeMatrix(srl)));
            }
            
            void coroot_radical_wrapper(expression_base::level l)
            { shared_root_datum rd(get<root_datum_value>());
              if (l==expression_base::no_value)
                return;
              latticetypes::WeightList scl
                (rd->val.beginSimpleCoroot(),rd->val.endSimpleCoroot());
              scl.insert(scl.end(),rd->val.beginRadical(),rd->val.endRadical());
              push_value(new matrix_value(atlas::latticetypes::LatticeMatrix(scl)));
            }
            
            void dual_datum_wrapper(expression_base::level l)
            { shared_root_datum rd(get<root_datum_value>());
              if (l!=expression_base::no_value)
                push_value(new root_datum_value
                  (rootdata::RootDatum(rd->val,tags::DualTag())));
            }
            
            void integrality_datum_wrapper(expression_base::level l)
            { shared_rational_vector lambda = get<rational_vector_value>();
              shared_root_datum rd(get<root_datum_value>());
              if (lambda->val.size()!=rd->val.rank())
              { std::ostringstream o;
                o << "Length " << lambda->val.size()
                  << " of rational vector differs from rank " << rd->val.rank();
                throw std::runtime_error(o.str());
            
              }
              if (l!=expression_base::no_value)
                push_value(new root_datum_value 
                  (rootdata::integrality_datum(rd->val,lambda->val)));
            }
            
            void integrality_points_wrapper(expression_base::level l)
            { shared_rational_vector lambda = get<rational_vector_value>();
              shared_root_datum rd(get<root_datum_value>());
              if (lambda->val.size()!=rd->val.rank())
              { std::ostringstream o;
                o << "Length " << lambda->val.size()
                  << " of rational vector differs from rank " << rd->val.rank();
                throw std::runtime_error(o.str());
            
              }
              if (l==expression_base::no_value)
                return;
            
              arithmetic::RationalList ipl =
                rootdata::integrality_points(rd->val,lambda->val);
              row_ptr result (new row_value(ipl.size()));
              for (size_t i=0; i<ipl.size(); ++i)
                result->val[i]=shared_value(new rat_value(ipl[i]));
              push_value(result);
            }
            
            std::pair<size_t,size_t> classify_involution
              (const latticetypes::LatticeMatrix& M)
            throw (std::bad_alloc, std::runtime_error)
            { size_t r=M.numRows();
              
              { if (M.numRows()!=r or M.numColumns()!=r) throw std::runtime_error
                  ("Involution should be a "+str(r)+"x"+str(r)+" matrix, got a "
              
                   +str(M.numRows())+"x"+str(M.numColumns())+" matrix");
                latticetypes::LatticeMatrix Id,Q(M);
                matrix::identityMatrix(Id,r); Q*=M; // $Q=M^2$
                if (!(Q==Id)) throw std::runtime_error
                    ("Given transformation is not an involution");
              
              }
              tori::RealTorus Tor(M);
              return std::make_pair(Tor.compactRank(),Tor.splitRank());
            }
            
            void classify_wrapper(expression_base::level l)
            { shared_matrix M(get<matrix_value>());
              std::pair<size_t,size_t> p=classify_involution(M->val);
              if (l==expression_base::no_value)
                return;
              push_value(new int_value(p.first)); // compact rank
              push_value(new int_value((M->val.numRows()-p.first-p.second)/2));
                // Complex rank
              push_value(new int_value(p.second)); // split rank
              if (l==expression_base::single_value)
                wrap_tuple(3);
            }
            
            lietype::Layout check_involution
             (const latticetypes::LatticeMatrix& M, const rootdata::RootDatum& rd)
             throw (std::bad_alloc, std::runtime_error)
            { size_t r=rd.rank(),s=rd.semisimpleRank();
              
              { if (M.numRows()!=r or M.numColumns()!=r) throw std::runtime_error
                  ("Involution should be a "+str(r)+"x"+str(r)+" matrix, got a "
              
                   +str(M.numRows())+"x"+str(M.numColumns())+" matrix");
                latticetypes::LatticeMatrix Id,Q(M);
                matrix::identityMatrix(Id,r); Q*=M; // $Q=M^2$
                if (!(Q==Id)) throw std::runtime_error
                    ("Given transformation is not an involution");
              
              }
            setutils::Permutation p(s);
              
              { rootdata::WRootIterator first=rd.beginSimpleRoot(), last=rd.endSimpleRoot();
                for (size_t i=0; i<s; ++i)
                { rootdata::Root alpha=M.apply(rd.simpleRoot(i));
                  size_t p_i=std::find(first,last,alpha)-first;
                  if (p_i<s) p[i]=p_i;
                  else throw std::runtime_error
                    ("Given transformation does not permute simple roots");
              
                }
                for (size_t i=0; i<s; ++i)
                  for (size_t j=0; j<s; ++j)
                    if (rd.cartan(p[i],p[j])!=rd.cartan(i,j)) throw std::runtime_error
                    ("Given transformation is not a root datum automorphism");
              }
            lietype::Layout result;
            lietype::LieType& type=result.d_type;
              lietype::InnerClassType& inner_class=result.d_inner;
              setutils::Permutation& pi=result.d_perm;
              
              { latticetypes::LatticeMatrix Cartan = rd.cartanMatrix();
                bitset::RankFlagsList comp =
                  dynkin::components(dynkin::DynkinDiagram(Cartan));
              
                type.reserve(comp.size()+r-s);
                inner_class.reserve(comp.size()+r-s); // certainly enough
                type = dynkin::Lie_type(Cartan,true,false,pi);
                  // no need to check validity of |Cartan|
                assert(type.size()==comp.size());
                size_t offset=0; // accumulated rank of simple factors seen
              
                for (size_t i=0; i<type.size(); ++i)
                { bool equal_rank=true;
                  size_t comp_rank = type[i].rank();
                  assert (comp_rank==comp[i].count());
                     // and |pi[j]| runs through |comp[i]| in following loop
                  for (size_t j=offset; j<offset+comp_rank; ++j) // traverse component
                    if (p[pi[j]]!=pi[j]) { equal_rank=false; break; }
                if (equal_rank) inner_class.push_back('c');
                    // identity on this component: compact component
                  else if(comp[i].test(p[pi[offset]]))
                     // (any) one root stays in |comp[i]|; unequal rank
                    inner_class.push_back(type[i].first=='D' and comp_rank%2==0 ? 'u' : 's');
                  else
                  { inner_class.push_back('C'); // record Complex component
                    
                    { size_t beta = p[pi[offset]];
                      
                      { size_t j, k;
                        for (j=offset+comp_rank,k=i+1; k<type.size(); ++k)
                          if (comp[k].test(beta))
                            break;
                          else
                            j+=type[k].second;
                      
                      
                        if (k==type.size())
                          throw std::logic_error("Non matching Complex factor");
                      
                      #ifndef NDEBUG
                        assert(type[k]==type[i]); // paired simple types for complex factor
                        for (size_t l=1; l<comp_rank; ++l)
                          assert(comp[k].test(p[pi[offset+l]]));
                              // image by |p| of remainder of |comp[i]| matches |comp[k]|
                      #endif
                      
                        if (k>i+1) // then we need to move component |k| down to |i+1|
                        {
                          while (j-->offset+comp_rank) // shift up intermediate components in |pi|
                            pi[j+comp_rank]=pi[j];
                      
                          bitset::RankFlags match_comp=comp[k];
                            // save component matching |comp[i]| under |p|
                      
                          while(k-->i+1) // shift intermediate simple factors
                          { type[k+1]=type[k]; comp[k+1]=comp[k]; }
                      
                          comp[i+1]=match_comp; // reinsert mathcing component after |comp[i]|
                        }
                      
                      }
                    
                      type[i+1]=type[i]; // duplicate factor |i|
                      for (size_t j=offset; j<offset+comp_rank; ++j)
                        pi[j+comp_rank]=p[pi[j]];
                         // reconstruct matching component in matching order
                    }
                    offset += comp_rank;
                    ++i; // skip over component |i|, loop will skip component |i+1|
                  }
              
                  offset += comp_rank;
                } // |for (i)|
              }
              if (r>s)
                
                { for (size_t k=0; k<r-s; ++k)
                    type.push_back(lietype::SimpleLieType('T',1));
                  latticetypes::WeightList b; matrix::initBasis(b,r);
                  latticetypes::WeightList simple_roots
                   (rd.beginSimpleRoot(),rd.endSimpleRoot());
                latticetypes::CoeffList ivf;
                  smithnormal::smithNormal(ivf,b.begin(),simple_roots);
                latticetypes::LatticeMatrix M_Smith(M,b); // express |M| on basis |b|
                  latticetypes::LatticeMatrix inv(M_Smith,s,s,r,r);
                    // involution on quotient by root lattice
                  std::pair<size_t,size_t> cl=classify_involution(inv);
                size_t& compact_rank=cl.first;
                  size_t& split_rank=cl.second;
                  size_t Complex_rank=(r-s-compact_rank-split_rank)/2;
                while (compact_rank-->0) inner_class.push_back('c');
                  while (Complex_rank-->0) inner_class.push_back('C');
                  while (split_rank-->0) inner_class.push_back('s');
                }
              return result;
            }
            
            void fix_involution_wrapper(expression_base::level l)
            { shared_matrix M(get<matrix_value>());
              shared_root_datum rd(get<root_datum_value>());
              lietype::Layout lo = check_involution(M->val,rd->val);
              if (l==expression_base::no_value)
                return;
            
              std::auto_ptr<complexredgp::ComplexReductiveGroup>
                G(new complexredgp::ComplexReductiveGroup(rd->val,M->val));
              push_value(new inner_class_value(G,lo));
            }
            
            void set_type_wrapper(expression_base::level l)
            { shared_string ict = get<string_value>();
                // and leave generators |gen| and type |lt|
              shared_value lt = *(execution_stack.end()-2);
              lietype::LieType& type=force<Lie_type_value>(lt.get())->val;
              lietype::Layout lo(type,transform_inner_class_type(ict->val.c_str(),type));
            
              quotient_basis_wrapper(expression_base::single_value); 
              shared_value basis = pop_value();
            
              push_value(lt); push_value(basis);
              root_datum_wrapper(expression_base::single_value);
              shared_root_datum rd = get<root_datum_value>();
            
              push_value(lt); push_value(basis);
              push_value(ict);
              based_involution_wrapper(expression_base::single_value);
              shared_matrix M = get<matrix_value>();
              if (l==expression_base::no_value)
                return; // bow out now all possible errors are passed
            
              std::auto_ptr<complexredgp::ComplexReductiveGroup>
                G(new complexredgp::ComplexReductiveGroup(rd->val,M->val));
              push_value(new inner_class_value(G,lo));
            }
            
            void set_inner_class_wrapper(expression_base::level l)
            { shared_string ict(get<string_value>());
              shared_root_datum rdv(get<root_datum_value>());
              const rootdata::RootDatum& rd=rdv->val;
            
              lietype::Layout lo;
              lo.d_type = dynkin::Lie_type(rd.cartanMatrix(),true,false,lo.d_perm);
               // get (permuted) type
              for (size_t i=rd.semisimpleRank(); i<rd.rank(); ++i) // if not semisimple
              { lo.d_type.push_back(lietype::SimpleLieType('T',1)); // add a torus factor
                lo.d_perm.push_back(i);
                  // and a fixed point of permutation, needed by |lietype::involution|
              }
              lo.d_inner=transform_inner_class_type(ict->val.c_str(),lo.d_type);
            
              push_value(rdv);
              coroot_radical_wrapper(expression_base::single_value);
              transpose_mat_wrapper(expression_base::single_value);
              shared_matrix M(get<matrix_value>());
              size_t r=lo.d_type.rank();
              assert(M->val.numRows()==r and M->val.numRows()==r);
            
              push_value(rdv);
              try
              { push_value(new matrix_value
                   (lietype::involution(lo).on_basis(M->val.columns())));
              }
              catch (std::runtime_error&) // relabel inexact division error
              { throw std::runtime_error 
                ("Inner class is not compatible with root datum");
            
              }
              fix_involution_wrapper(l);
            }
            
            void distinguished_involution_wrapper(expression_base::level l)
            { shared_inner_class G(get<inner_class_value>());
              if (l!=expression_base::no_value)
                push_value(new matrix_value(G->val.distinguished()));
            }
            
            void root_datum_of_inner_class_wrapper(expression_base::level l)
            { shared_inner_class G(get<inner_class_value>());
              if (l!=expression_base::no_value)
                push_value(new root_datum_value(G->val.rootDatum()));
            }
            
            void dual_inner_class_wrapper(expression_base::level l)
            { shared_inner_class G(get<inner_class_value>());
              if (l!=expression_base::no_value)
                push_value(new inner_class_value(*G,tags::DualTag()));
            }
            
            void push_name_list(const realform_io::Interface& interface)
            { row_ptr result(new row_value(0));
              for (size_t i=0; i<interface.numRealForms(); ++i)
                result->val.push_back
                  (shared_value(new string_value(interface.typeName(i))));
              push_value(result);
            }
            
            void form_names_wrapper(expression_base::level l)
            { shared_inner_class G(get<inner_class_value>());
              if (l!=expression_base::no_value)
                push_name_list(G->interface);
            }
            
            void dual_form_names_wrapper(expression_base::level l)
            { shared_inner_class G(get<inner_class_value>());
              if (l!=expression_base::no_value)
                push_name_list(G->dual_interface);
            }
            
            void block_sizes_wrapper(expression_base::level l)
            { shared_inner_class G(get<inner_class_value>());
              if (l==expression_base::no_value)
                return;
              matrix_ptr M(new matrix_value 
                (latticetypes::LatticeMatrix(G->val.numRealForms()
                                            ,G->val.numDualRealForms())
                ));
              for (size_t i = 0; i < M->val.numRows(); ++i)
                for (size_t j = 0; j < M->val.numColumns(); ++j)
                  M->val(i,j) =
                  G->val.block_size(G->interface.in(i),G->dual_interface.in(j));
              push_value(M);
            }
            
            void occurrence_matrix_wrapper(expression_base::level l)
            { shared_inner_class G(get<inner_class_value>());
              if (l==expression_base::no_value)
                return;
              size_t nr=G->val.numRealForms();
              size_t nc=G->val.numCartanClasses();
              matrix_ptr M(new matrix_value(latticetypes::LatticeMatrix(nr,nc)));
              for (size_t i=0; i<nr; ++i)
              { bitmap::BitMap b=G->val.Cartan_set(G->interface.in(i));
                for (size_t j=0; j<nc; ++j)
                  M->val(i,j)= b.isMember(j) ? 1 : 0;
              }
              push_value(M);
            }
            
            void dual_occurrence_matrix_wrapper(expression_base::level l)
            { shared_inner_class G(get<inner_class_value>());
              if (l==expression_base::no_value)
                return;
              size_t nr=G->val.numDualRealForms();
              size_t nc=G->val.numCartanClasses();
              matrix_ptr M(new matrix_value(latticetypes::LatticeMatrix(nr,nc)));
              for (size_t i=0; i<nr; ++i)
              { bitmap::BitMap b=G->val.dual_Cartan_set(G->dual_interface.in(i));
                for (size_t j=0; j<nc; ++j)
                  M->val(i,j)= b.isMember(j) ? 1 : 0;
              }
              push_value(M);
            }
            
            void real_form_wrapper(expression_base::level l)
            { shared_int i(get<int_value>());
              shared_inner_class G(get<inner_class_value>());
              if (size_t(i->val)>=G->val.numRealForms())
                throw std::runtime_error ("Illegal real form number: "+str(i->val));
            
              if (l!=expression_base::no_value)
                push_value(new real_form_value(*G,G->interface.in(i->val)));
            }
            
            void quasisplit_form_wrapper(expression_base::level l)
            { shared_inner_class G(get<inner_class_value>());
              if (l!=expression_base::no_value)
                push_value(new real_form_value(*G,G->val.quasisplit()));
            }
            
            void class_of_real_group_wrapper(expression_base::level l)
            { shared_real_form rf(get<real_form_value>());
              if (l!=expression_base::no_value)
                push_value(new inner_class_value(rf->parent));
            }
            
            void components_rank_wrapper(expression_base::level l)
            { shared_real_form R(get<real_form_value>());
              if (l!=expression_base::no_value)
              { const latticetypes::ComponentList c=R->val.dualComponentReps();
                push_value(new int_value(c.size()));
              }
            }
            
            void count_Cartans_wrapper(expression_base::level l)
            { shared_real_form rf(get<real_form_value>());
              if (l!=expression_base::no_value)
                push_value(new int_value(rf->val.numCartan()));
            }
            
            void KGB_size_wrapper(expression_base::level l)
            { shared_real_form rf(get<real_form_value>());
              if (l!=expression_base::no_value)
                push_value(new int_value(rf->val.KGB_size()));
            }
            
            void Cartan_order_matrix_wrapper(expression_base::level l)
            { shared_real_form rf(get<real_form_value>());
              if (l==expression_base::no_value)
                return;
              size_t n=rf->val.numCartan();
              matrix_ptr M(new matrix_value(latticetypes::LatticeMatrix(n,n,0)));
              const poset::Poset& p = rf->val.complexGroup().Cartan_ordering();
              for (size_t i=0; i<n; ++i)
                for (size_t j=i; j<n; ++j)
                  if (p.lesseq(i,j)) M->val(i,j)=1;
            
              push_value(M);
            }
            
            void dual_real_form_wrapper(expression_base::level l)
            { shared_int i(get<int_value>());
              shared_inner_class G(get<inner_class_value>());
              if (size_t(i->val)>=G->val.numDualRealForms())
                throw std::runtime_error ("Illegal dual real form number: "+str(i->val));
            
              if (l!=expression_base::no_value)
                push_value(new dual_real_form_value(*G,G->dual_interface.in(i->val)));
            }
            
            void dual_quasisplit_form_wrapper(expression_base::level l)
            { shared_inner_class G(get<inner_class_value>());
              if (l!=expression_base::no_value)
                push_value(new dual_real_form_value(*G,G->dual.quasisplit()));
            }
            
            void real_form_from_dual_wrapper(expression_base::level l)
            { shared_dual_real_form d(get<dual_real_form_value>());
              if (l!=expression_base::no_value)
                push_value(new real_form_value
                             (inner_class_value(d->parent,tags::DualTag())
                             ,d->val.realForm()));
            }
            
            void Cartan_class_wrapper(expression_base::level l)
            { shared_int i(get<int_value>());
              shared_real_form rf(get<real_form_value>());
              if (size_t(i->val)>=rf->val.numCartan())
                throw std::runtime_error
                ("Illegal Cartan class number: "+str(i->val)
            
                +", this real form only has "+str(rf->val.numCartan())+" of them");
              bitmap::BitMap cs=rf->val.Cartan_set();
              if (l!=expression_base::no_value)
                push_value(new Cartan_class_value(rf->parent,cs.n_th(i->val)));
            }
            
            void most_split_Cartan_wrapper(expression_base::level l)
            { shared_real_form rf(get<real_form_value>());
              if (l!=expression_base::no_value)
                push_value(new Cartan_class_value(rf->parent,rf->val.mostSplit()));
            }
            
            void print_Cartan_info_wrapper(expression_base::level l)
            { shared_Cartan_class cc(get<Cartan_class_value>());
            
              prettyprint::printTorusType(*output_stream,cc->val.fiber().torus())
              << std::endl;
            
              *output_stream << "twisted involution orbit size: " << cc->val.orbitSize()
               << std::endl;
            
              const rootdata::RootSystem& rs=cc->parent.val.rootDatum();
            
            // print type of imaginary root system
              lietype::LieType ilt = rs.Lie_type(cc->val.simpleImaginary());
            
              if (ilt.size() == 0)
                *output_stream << "imaginary root system is empty" << std::endl;
              else
                *output_stream << "imaginary root system: " << ilt << std::endl;
            
            // print type of real root system
            
              lietype::LieType rlt = rs.Lie_type(cc->val.simpleReal());
            
              if (rlt.size() == 0)
                *output_stream << "real root system is empty" << std::endl;
              else
                *output_stream << "real root system: " << rlt << std::endl;
            
            // print type of complex root system
            
              lietype::LieType clt = rs.Lie_type(cc->val.simpleComplex());
            
              if (clt.size() == 0)
                *output_stream << "complex factor is empty" << std::endl;
              else
                *output_stream << "complex factor: " << clt << std::endl;
            
              if (l==expression_base::single_value)
                wrap_tuple(0);
            }
            
            void real_forms_of_Cartan_wrapper(expression_base::level l)
            { shared_Cartan_class cc(get<Cartan_class_value>());
              const inner_class_value& ic=cc->parent;
              if (l==expression_base::no_value)
                return;
              row_ptr result  (new row_value(cc->val.numRealForms()));
              for (size_t i=0,k=0; i<ic.val.numRealForms(); ++i)
              { bitmap::BitMap b(ic.val.Cartan_set(ic.interface.in(i)));
                if (b.isMember(cc->number))
                  result->val[k++] =
            	shared_value(new real_form_value(ic,ic.interface.in(i)));
              }
              push_value(result);
            }
            
            void dual_real_forms_of_Cartan_wrapper(expression_base::level l)
            { shared_Cartan_class cc(get<Cartan_class_value>());
              const inner_class_value& ic=cc->parent;
              if (l==expression_base::no_value)
                return;
              row_ptr result  (new row_value(cc->val.numDualRealForms()));
              for (size_t i=0,k=0; i<ic.val.numDualRealForms(); ++i)
              { bitmap::BitMap b(ic.val.dual_Cartan_set(ic.dual_interface.in(i)));
                if (b.isMember(cc->number))
                  result->val[k++] =
            	shared_value(new dual_real_form_value(ic,ic.dual_interface.in(i)));
              }
              push_value(result);
            }
            
            void fiber_part_wrapper(expression_base::level l)
            { shared_real_form rf(get<real_form_value>());
              shared_Cartan_class cc(get<Cartan_class_value>());
              if (&rf->parent.val!=&cc->parent.val)
                throw std::runtime_error
                ("Inner class mismatch between real form and Cartan class");
            
              bitmap::BitMap b(cc->parent.val.Cartan_set(rf->val.realForm()));
              if (!b.isMember(cc->number))
                throw std::runtime_error
                ("Cartan class not defined for this real form");
            
              if (l==expression_base::no_value)
                return;
            
              const partition::Partition& pi = cc->val.fiber().weakReal();
              const realform::RealFormList rf_nr=
                 cc->parent.val.realFormLabels(cc->number);
                 // translate part number of |pi| to real form
              row_ptr result (new row_value(0)); // cannot predict exact size here
              for (size_t i=0; i<pi.size(); ++i)
                if (rf_nr[pi(i)] == rf->val.realForm())
                  result->val.push_back(shared_value(new int_value(i)));
              push_value(result);
            }
            
            void print_gradings_wrapper(expression_base::level l)
            { shared_real_form rf(get<real_form_value>());
            shared_Cartan_class cc(get<Cartan_class_value>());
              if (&rf->parent.val!=&cc->parent.val)
                throw std::runtime_error
                ("Inner class mismatch between real form and Cartan class");
            
              bitmap::BitMap b(cc->parent.val.Cartan_set(rf->val.realForm()));
              if (!b.isMember(cc->number))
                throw std::runtime_error
                ("Cartan class not defined for this real form");
            
            
              const partition::Partition& pi = cc->val.fiber().weakReal();
              const realform::RealFormList rf_nr=
                 cc->parent.val.realFormLabels(cc->number);
                 // translate part number of |pi| to real form
            
              const rootdata::RootList& si = cc->val.fiber().simpleImaginary();
              // simple imaginary roots
            
              latticetypes::LatticeMatrix cm;
              setutils::Permutation sigma;
            
            { cm=cc->parent.val.rootDatum().cartanMatrix(si);
              dynkin::DynkinDiagram d(cm); sigma = dynkin::bourbaki(d);
            }
              
              { *output_stream << "Imaginary root system is ";
                if (si.size()==0) *output_stream<<"empty.\n";
                else
                { lietype::LieType t = dynkin::Lie_type(cm);
              
                  *output_stream << "of type " << t << ", with simple root"
                            << (si.size()==1 ? " " : "s ");
                  for (size_t i=0; i<si.size(); ++i)
                    *output_stream << si[sigma[i]] << (i<si.size()-1 ? "," : ".\n");
                }
              }
              
              { bool first=true; std::ostringstream os;
                for (size_t i=0; i<pi.size(); ++i)
                  if ( rf_nr[pi(i)] == rf->val.realForm())
                  { os << ( first ? first=false,'[' : ',');
                    gradings::Grading gr=cc->val.fiber().grading(i);
                    gr.permute(sigma);
                    prettyprint::prettyPrint(os,gr,si.size());
                  }
                os << "]" << std::endl;
                ioutils::foldLine(*output_stream,os.str(),"",",");
              }
              if (l==expression_base::single_value)
                wrap_tuple(0);
            }
            
            void print_realweyl_wrapper(expression_base::level l)
            { shared_Cartan_class cc(get<Cartan_class_value>());
              shared_real_form rf(get<real_form_value>());
            
              if (&rf->parent.val!=&cc->parent.val)
                throw std::runtime_error 
                ("Inner class mismatch between arguments");
            
              bitmap::BitMap b(rf->parent.val.Cartan_set(rf->val.realForm()));
              if (not b.isMember(cc->number))
                throw std::runtime_error 
                ("Cartan class not defined for real form");
            
            
              realredgp_io::printRealWeyl (*output_stream,rf->val,cc->number);
            
              if (l==expression_base::single_value)
                wrap_tuple(0);
            }
            
            
            void print_strongreal_wrapper(expression_base::level l)
            { shared_Cartan_class cc(get<Cartan_class_value>());
              shared_real_form rf(get<real_form_value>());
            
              if (&rf->parent.val!=&cc->parent.val)
                throw std::runtime_error 
                ("Inner class mismatch between arguments");
            
              bitmap::BitMap b(rf->parent.val.Cartan_set(rf->val.realForm()));
              if (not b.isMember(cc->number))
                throw std::runtime_error 
                ("Cartan class not defined for real form");
            
            
              realredgp_io::printStrongReal
                (*output_stream,rf->val,rf->parent.interface,cc->number);
            
              if (l==expression_base::single_value)
                wrap_tuple(0);
            }
            
            void print_block_wrapper(expression_base::level l)
            { shared_dual_real_form drf(get<dual_real_form_value>());
              shared_real_form rf(get<real_form_value>());
            
              if (&rf->parent.val!=&drf->parent.val)
                throw std::runtime_error 
                ("Inner class mismatch between real form and dual real form");
            
              bitmap::BitMap b(rf->parent.val.dual_Cartan_set(drf->val.realForm()));
              if (!b.isMember(rf->val.mostSplit()))
                throw std::runtime_error 
                ("Real form and dual real form are incompatible");
            
            
              blocks::Block block(rf->parent.val
                                 ,rf->val.realForm(),drf->val.realForm());
              block_io::printBlock(*output_stream,block);
            
              if (l==expression_base::single_value)
                wrap_tuple(0);
            }
            
            void print_blockd_wrapper(expression_base::level l)
            { shared_dual_real_form drf(get<dual_real_form_value>());
              shared_real_form rf(get<real_form_value>());
            
              if (&rf->parent.val!=&drf->parent.val)
                throw std::runtime_error 
                ("Inner class mismatch between real form and dual real form");
            
              bitmap::BitMap b(rf->parent.val.dual_Cartan_set(drf->val.realForm()));
              if (!b.isMember(rf->val.mostSplit()))
                throw std::runtime_error 
                ("Real form and dual real form are incompatible");
            
            
              blocks::Block block(rf->parent.val
                                 ,rf->val.realForm(),drf->val.realForm());
              block_io::printBlockD(*output_stream,block);
            
              if (l==expression_base::single_value)
                wrap_tuple(0);
            }
            
            
            void print_blocku_wrapper(expression_base::level l)
            { shared_dual_real_form drf(get<dual_real_form_value>());
              shared_real_form rf(get<real_form_value>());
            
              if (&rf->parent.val!=&drf->parent.val)
                throw std::runtime_error 
                ("Inner class mismatch between real form and dual real form");
            
              bitmap::BitMap b(rf->parent.val.dual_Cartan_set(drf->val.realForm()));
              if (!b.isMember(rf->val.mostSplit()))
                throw std::runtime_error 
                ("Real form and dual real form are incompatible");
            
            
              blocks::Block block(rf->parent.val
                                 ,rf->val.realForm(),drf->val.realForm());
              block_io::printBlockU(*output_stream,block);
            
              if (l==expression_base::single_value)
                wrap_tuple(0);
            }
            
            void print_blockstabilizer_wrapper(expression_base::level l)
            { shared_Cartan_class cc(get<Cartan_class_value>());
              shared_dual_real_form drf(get<dual_real_form_value>());
              shared_real_form rf(get<real_form_value>());
            
              if (&rf->parent.val!=&drf->parent.val or
                  &rf->parent.val!=&cc->parent.val)
                throw std::runtime_error 
                ("Inner class mismatch between arguments");
            
              bitmap::BitMap b(rf->parent.val.Cartan_set(rf->val.realForm()));
              b &= bitmap::BitMap(rf->parent.val.dual_Cartan_set(drf->val.realForm()));
              if (not b.isMember(cc->number))
                throw std::runtime_error 
                ("Cartan class not defined for both real forms");
            
            
            
              realredgp_io::printBlockStabilizer
               (*output_stream,rf->val,cc->number,drf->val.realForm());
            
              if (l==expression_base::single_value)
                wrap_tuple(0);
            }
            
            void print_KGB_wrapper(expression_base::level l)
            { shared_real_form rf(get<real_form_value>());
            
              *output_stream
                << "kgbsize: " << rf->val.KGB_size() << std::endl;
              kgb::KGB kgb(rf->val);
              kgb_io::var_print_KGB(*output_stream,rf->val.complexGroup(),kgb);
            
              if (l==expression_base::single_value)
                wrap_tuple(0);
            }
            
            void print_KL_basis_wrapper(expression_base::level l)
            { shared_dual_real_form drf(get<dual_real_form_value>());
              shared_real_form rf(get<real_form_value>());
            
              if (&rf->parent.val!=&drf->parent.val)
                throw std::runtime_error 
                ("Inner class mismatch between real form and dual real form");
            
              bitmap::BitMap b(rf->parent.val.dual_Cartan_set(drf->val.realForm()));
              if (!b.isMember(rf->val.mostSplit()))
                throw std::runtime_error 
                ("Real form and dual real form are incompatible");
            
            
              blocks::Block block(rf->parent.val
                                 ,rf->val.realForm(),drf->val.realForm());
              klsupport::KLSupport kls(block); kls.fill();
            
              kl::KLContext klc(kls); klc.fill();
            
              *output_stream
                << "Full list of non-zero Kazhdan-Lusztig-Vogan polynomials:\n\n";
              kl_io::printAllKL(*output_stream,klc);
            
              if (l==expression_base::single_value)
                wrap_tuple(0);
            }
            
            void print_prim_KL_wrapper(expression_base::level l)
            { shared_dual_real_form drf(get<dual_real_form_value>());
              shared_real_form rf(get<real_form_value>());
            
              if (&rf->parent.val!=&drf->parent.val)
                throw std::runtime_error 
                ("Inner class mismatch between real form and dual real form");
            
              bitmap::BitMap b(rf->parent.val.dual_Cartan_set(drf->val.realForm()));
              if (!b.isMember(rf->val.mostSplit()))
                throw std::runtime_error 
                ("Real form and dual real form are incompatible");
            
            
              blocks::Block block(rf->parent.val
                                 ,rf->val.realForm(),drf->val.realForm());
              klsupport::KLSupport kls(block); kls.fill();
            
              kl::KLContext klc(kls); klc.fill();
            
              *output_stream
                << "Non-zero Kazhdan-Lusztig-Vogan polynomials for primitive pairs:\n\n";
              kl_io::printPrimitiveKL(*output_stream,klc);
            
              if (l==expression_base::single_value)
                wrap_tuple(0);
            }
            
            void print_KL_list_wrapper(expression_base::level l)
            { shared_dual_real_form drf(get<dual_real_form_value>());
              shared_real_form rf(get<real_form_value>());
            
              if (&rf->parent.val!=&drf->parent.val)
                throw std::runtime_error 
                ("Inner class mismatch between real form and dual real form");
            
              bitmap::BitMap b(rf->parent.val.dual_Cartan_set(drf->val.realForm()));
              if (!b.isMember(rf->val.mostSplit()))
                throw std::runtime_error 
                ("Real form and dual real form are incompatible");
            
            
              blocks::Block block(rf->parent.val
                                 ,rf->val.realForm(),drf->val.realForm());
              klsupport::KLSupport kls(block); kls.fill();
            
              kl::KLContext klc(kls); klc.fill();
            
              kl_io::printKLList(*output_stream,klc);
            
              if (l==expression_base::single_value)
                wrap_tuple(0);
            }
            
            void print_W_cells_wrapper(expression_base::level l)
            { shared_dual_real_form drf(get<dual_real_form_value>());
              shared_real_form rf(get<real_form_value>());
            
              if (&rf->parent.val!=&drf->parent.val)
                throw std::runtime_error 
                ("Inner class mismatch between real form and dual real form");
            
              bitmap::BitMap b(rf->parent.val.dual_Cartan_set(drf->val.realForm()));
              if (!b.isMember(rf->val.mostSplit()))
                throw std::runtime_error 
                ("Real form and dual real form are incompatible");
            
            
              blocks::Block block(rf->parent.val
                                 ,rf->val.realForm(),drf->val.realForm());
              klsupport::KLSupport kls(block); kls.fill();
            
              kl::KLContext klc(kls); klc.fill();
            
              wgraph::WGraph wg(klc.rank()); kl::wGraph(wg,klc);
              wgraph::DecomposedWGraph dg(wg);
            
              wgraph_io::printWDecomposition(*output_stream,dg);
            
              if (l==expression_base::single_value)
                wrap_tuple(0);
            }
            
            void print_W_graph_wrapper(expression_base::level l)
            { shared_dual_real_form drf(get<dual_real_form_value>());
              shared_real_form rf(get<real_form_value>());
            
              if (&rf->parent.val!=&drf->parent.val)
                throw std::runtime_error 
                ("Inner class mismatch between real form and dual real form");
            
              bitmap::BitMap b(rf->parent.val.dual_Cartan_set(drf->val.realForm()));
              if (!b.isMember(rf->val.mostSplit()))
                throw std::runtime_error 
                ("Real form and dual real form are incompatible");
            
            
              blocks::Block block(rf->parent.val
                                 ,rf->val.realForm(),drf->val.realForm());
              klsupport::KLSupport kls(block); kls.fill();
            
              kl::KLContext klc(kls); klc.fill();
            
              wgraph::WGraph wg(klc.rank()); kl::wGraph(wg,klc);
            
              wgraph_io::printWGraph(*output_stream,wg);
            
              if (l==expression_base::single_value)
                wrap_tuple(0);
            }
 }

void initialise_builtin_types()
{ 
  install_function(Lie_type_wrapper,"Lie_type","(string->LieType)");
  
  install_function(Cartan_matrix_wrapper,"Cartan_matrix","(LieType->mat)");
  install_function(type_of_Cartan_matrix_wrapper
  		,"Cartan_matrix_type","(mat->LieType,vec)");
  install_function(Lie_rank_wrapper,"Lie_rank","(LieType->int)");
  install_function(semisimple_rank_wrapper,"semisimple_rank","(LieType->int)");
  install_function(nr_factors_wrapper,"nr_factors","(LieType->int)");
  
  install_function(Smith_Cartan_wrapper,"Smith_Cartan","(LieType->mat,vec)");
  install_function(filter_units_wrapper,"filter_units","(mat,vec->mat,vec)");
  install_function(ann_mod_wrapper,"ann_mod","(mat,int->mat)");
  install_function(replace_gen_wrapper,"replace_gen",
  		"((mat,vec),mat->mat)");
  install_function(basic_involution_wrapper,"involution",
  		"(LieType,string->mat)");
  install_function(based_involution_wrapper,"involution",
  		"(LieType,mat,string->mat)");
  
  install_function(type_of_root_datum_wrapper,"Lie_type"
                  ,"(RootDatum->LieType)");
  install_function(root_datum_wrapper,"root_datum","(LieType,mat->RootDatum)");
  install_function(raw_root_datum_wrapper,
                   "root_datum","([vec],[vec],int->RootDatum)");
  install_function(quotient_basis_wrapper
  		,"quotient_basis","(LieType,[ratvec]->mat)");
  install_function(quotient_datum_wrapper
  		,"root_datum","(LieType,[ratvec]->RootDatum)");
  install_function(simply_connected_datum_wrapper
  		,"simply_connected","(LieType->RootDatum)");
  install_function(adjoint_datum_wrapper, "adjoint","(LieType->RootDatum)");
  install_function(SL_wrapper,"SL","(int->RootDatum)");
  install_function(GL_wrapper,"GL","(int->RootDatum)");
  install_function(simple_roots_wrapper,"simple_roots","(RootDatum->mat)");
  install_function(simple_coroots_wrapper,"simple_coroots","(RootDatum->mat)");
  install_function(datum_Cartan_wrapper,"Cartan_matrix"
  		,"(RootDatum->mat)");
  install_function(roots_wrapper,"roots","(RootDatum->mat)");
  install_function(coroots_wrapper,"coroots","(RootDatum->mat)");
  install_function(root_coradical_wrapper,"root_coradical","(RootDatum->mat)");
  install_function(coroot_radical_wrapper,"coroot_radical","(RootDatum->mat)");
  install_function(dual_datum_wrapper,"dual","(RootDatum->RootDatum)");
  install_function(integrality_datum_wrapper
                  ,"integrality_datum","(RootDatum,ratvec->RootDatum)");
  install_function(integrality_points_wrapper
                  ,"integrality_points","(RootDatum,ratvec->[rat])");
  
  install_function(classify_wrapper,"classify_involution"
                  ,"(mat->int,int,int)");
  install_function(fix_involution_wrapper,"inner_class"
                  ,"(RootDatum,mat->InnerClass)");
  install_function(set_type_wrapper,"inner_class"
                  ,"(LieType,[ratvec],string->InnerClass)");
  install_function(set_inner_class_wrapper,"inner_class"
                  ,"(RootDatum,string->InnerClass)");
  install_function(distinguished_involution_wrapper,"distinguished_involution"
                  ,"(InnerClass->mat)");
  install_function(root_datum_of_inner_class_wrapper,"root_datum"
                  ,"(InnerClass->RootDatum)");
  install_function(dual_inner_class_wrapper,"dual"
                  ,"(InnerClass->InnerClass)");
  install_function(form_names_wrapper,"form_names"
                  ,"(InnerClass->[string])");
  install_function(dual_form_names_wrapper,"dual_form_names"
                  ,"(InnerClass->[string])");
  install_function(block_sizes_wrapper,"block_sizes"
                  ,"(InnerClass->mat)");
  install_function(occurrence_matrix_wrapper,"occurrence_matrix"
                  ,"(InnerClass->mat)");
  install_function(dual_occurrence_matrix_wrapper,"dual_occurrence_matrix"
                  ,"(InnerClass->mat)");
  
  install_function(real_form_wrapper,"real_form","(InnerClass,int->RealForm)");
  install_function(quasisplit_form_wrapper,"quasisplit_form"
  		,"(InnerClass->RealForm)");
  install_function(class_of_real_group_wrapper
                  ,"inner_class","(RealForm->InnerClass)");
  install_function(components_rank_wrapper,"components_rank","(RealForm->int)");
  install_function(count_Cartans_wrapper,"count_Cartans","(RealForm->int)");
  install_function(KGB_size_wrapper,"KGB_size","(RealForm->int)");
  install_function(Cartan_order_matrix_wrapper,"Cartan_order"
  					    ,"(RealForm->mat)");
  install_function(dual_real_form_wrapper,"dual_real_form"
  				       ,"(InnerClass,int->DualRealForm)");
  install_function(dual_quasisplit_form_wrapper,"dual_quasisplit_form"
  		,"(InnerClass->DualRealForm)");
  install_function(real_form_from_dual_wrapper,"real_form"
  				  ,"(DualRealForm->RealForm)");
  install_function(Cartan_class_wrapper,"Cartan_class"
  		,"(RealForm,int->CartanClass)");
  install_function(most_split_Cartan_wrapper,"most_split_Cartan"
  		,"(RealForm->CartanClass)");
  install_function(print_Cartan_info_wrapper,"print_Cartan_info"
  		,"(CartanClass->)");
  install_function(real_forms_of_Cartan_wrapper,"real_forms"
  		,"(CartanClass->[RealForm])");
  install_function(dual_real_forms_of_Cartan_wrapper,"dual_real_forms"
  		,"(CartanClass->[DualRealForm])");
  install_function(fiber_part_wrapper,"fiber_part"
  		,"(CartanClass,RealForm->[int])");
  install_function(print_gradings_wrapper,"print_gradings"
  		,"(CartanClass,RealForm->)");
  install_function(print_realweyl_wrapper,"print_real_Weyl"
  		,"(RealForm,CartanClass->)");
  install_function(print_strongreal_wrapper,"print_strong_real"
  		,"(RealForm,CartanClass->)");
  install_function(print_block_wrapper,"print_block"
  		,"(RealForm,DualRealForm->)");
  install_function(print_blocku_wrapper,"print_blocku"
  		,"(RealForm,DualRealForm->)");
  install_function(print_blockd_wrapper,"print_blockd"
  		,"(RealForm,DualRealForm->)");
  install_function(print_blockstabilizer_wrapper,"print_blockstabilizer"
  		,"(RealForm,DualRealForm,CartanClass->)");
  install_function(print_KGB_wrapper,"print_KGB"
  		,"(RealForm->)");
  install_function(print_KL_basis_wrapper,"print_KL_basis"
  		,"(RealForm,DualRealForm->)");
  install_function(print_prim_KL_wrapper,"print_prim_KL"
  		,"(RealForm,DualRealForm->)");
  install_function(print_KL_list_wrapper,"print_KL_list"
  		,"(RealForm,DualRealForm->)");
  install_function(print_W_cells_wrapper,"print_W_cells"
  		,"(RealForm,DualRealForm->)");
  install_function(print_W_graph_wrapper,"print_W_graph"
  		,"(RealForm,DualRealForm->)");
  
  { static type_expr Lie_type_type(complex_lie_type_type);
    coercion(str_type,Lie_type_type,"LT",Lie_type_coercion);
  }
}

void Lie_type_value::add_simple_factor (char c,size_t rank)
   throw(std::bad_alloc,std::runtime_error)
{ static const std::string types=atlas::lietype::typeLetters; // |"ABCDEFGT"|
  size_t t=types.find(c);
  if (t==std::string::npos)
    throw std::runtime_error(std::string("Invalid type letter '")+c+'\'');

  static const size_t lwb[]={1,2,2,4,6,4,2,0};
  const size_t r=constants::RANK_MAX;
  static const size_t upb[]={r,r,r,r,8,4,2,r};
  if (rank<lwb[t])
    throw std::runtime_error("Too small rank "+str(rank)+" for Lie type "+c);

  if (rank>upb[t])
    if (upb[t]!=r)
      throw std::runtime_error("Too large rank "+str(rank)+" for Lie type "+c);

    else
      throw std::runtime_error 
      ("Rank "+str(rank)+" exceeds implementation limit "+str(r));

  if (c=='T')
    while (rank-->0) val.push_back(lietype::SimpleLieType('T',1));
  else
    val.push_back(lietype::SimpleLieType(c,rank));
}

void Lie_type_value::print(std::ostream& out) const
{ if (val.empty()) out << "empty Lie type";
  else
    out << "Lie type '" << val << '\'';
}

void root_datum_value::print(std::ostream& out) const
{ Lie_type_value type(type_of_datum(val));
  out << (val.isSimplyConnected() ? "simply connected " : "")
    << (val.isAdjoint() ? "adjoint " : "")
      << "root datum of " << type;
}

inner_class_value::inner_class_value(const inner_class_value& v)
: val(v.val), dual(v.dual), ref_count(v.ref_count)
, type_info(v.type_info)
, interface(v.interface), dual_interface(v.dual_interface)
{ ++ref_count; }

inner_class_value::~inner_class_value()
{ if (--ref_count==0) { delete &val; delete &dual; delete &ref_count;} }

inner_class_value::inner_class_value
  (std::auto_ptr<complexredgp::ComplexReductiveGroup> g,
   const lietype::Layout& lo)
: val(*g)
, dual(*new complexredgp::ComplexReductiveGroup(*g,tags::DualTag()))
, ref_count(*new size_t(1))
, type_info(lo), interface(*g,lo), dual_interface(*g,lo,tags::DualTag())
 { g.release(); } 

inner_class_value::inner_class_value(const inner_class_value& v,tags::DualTag)
: val(v.dual), dual(v.val)
, ref_count(v.ref_count)
,type_info(v.type_info)
, interface(v.dual_interface), dual_interface(v.interface)
{ ++ref_count;
type_info.d_type=lietype::dual_type(type_info.d_type);
  type_info.d_inner=lietype::dual_type(type_info.d_inner,type_info.d_type);
// |d_perm| remains identical to |v.d_perm|
}

void inner_class_value::print(std::ostream& out) const
{ out << "Complex reductive group of type " << type_info.d_type 
      << ", with involution defining\n"
         "inner class of type '" << type_info.d_inner 
      << "', with " << val.numRealForms()  << " real "
      << (val.numRealForms()==1 ? "form" : "forms")  << " and "
      << val.numDualRealForms()  << " dual real "
      << (val.numDualRealForms()==1 ? "form" : "forms");
}

void real_form_value::print(std::ostream& out) const
{ if (val.isCompact()) out << "compact ";
  if (val.isQuasisplit())
    out << (val.isSplit() ? "" : "quasi") << "split ";
  out << "real form '" 
  << parent.interface.typeName(parent.interface.out(val.realForm())) 
  << "', defining a"
  << (val.isConnected() ? " connected" : " disconnected" ) 
  << " real group" ;
}

real_form_value::real_form_value(inner_class_value p,realform::RealForm f
				,tags::DualTag)
: parent(p), val(p.dual,f)
{}

void dual_real_form_value::print(std::ostream& out) const
{ out << (val.isQuasisplit() ? "quasisplit " : "")
  << "dual real form '" 
  << parent.dual_interface.typeName
    (parent.dual_interface.out(val.realForm())) 
  << "'";
}

Cartan_class_value::Cartan_class_value(const inner_class_value& p,size_t cn)
: parent(p),number(cn),val(p.val.cartan(cn))
{ if (cn>=p.val.numCartanClasses()) throw std::runtime_error
  (std::string("Cartan class number ")+str(cn)+" does not (currently) exist");
}

void Cartan_class_value::print(std::ostream& out) const
{ out << "Cartan class #" << number << ", occurring for " 
  << val.numRealForms() << " real "
  << (val.numRealForms()==1 ? "form" : "forms") << " and for "
  << val.numDualRealForms() << " dual real "
  << (val.numDualRealForms()==1 ? "form" : "forms");
}

} }

