
#ifndef BUILT_IN_TYPES_H
#define BUILT_IN_TYPES_H


#include "evaluator.h"

#include "lietype_fwd.h"

#include "rootdata.h"

#include "complexredgp.h"

#include "realform_io.h"

#include "realredgp.h"

#include "cartanclass.h"

namespace atlas { namespace interpreter {

struct Lie_type_value : public value_base
{ lietype::LieType val;

  Lie_type_value() : val()  {}
    // default constructor, produces empty type
  Lie_type_value(lietype::LieType t) : val(t) {}
    // constructor from already validated Lie type

  virtual void print(std::ostream& out) const;
  Lie_type_value* clone() const { return new Lie_type_value(*this); }
  static const char* name() { return "Lie type"; }

  void add_simple_factor (char,size_t)
    throw(std::bad_alloc,std::runtime_error); // grow
private:
  Lie_type_value(const Lie_type_value& v) : val(v.val) {}
    // copy constructor, used by |clone|
};

typedef std::auto_ptr<Lie_type_value> Lie_type_ptr;
typedef std::tr1::shared_ptr<Lie_type_value> shared_Lie_type;

struct root_datum_value : public value_base
{ rootdata::RootDatum val;

  root_datum_value(const rootdata::RootDatum v) : val(v)  {}
  virtual void print(std::ostream& out) const;
  root_datum_value* clone() const { return new root_datum_value(*this); }
  static const char* name() { return "root datum"; }
private:
  root_datum_value(const root_datum_value& v) : val(v.val) {}
};

typedef std::auto_ptr<root_datum_value> root_datum_ptr;
typedef std::tr1::shared_ptr<root_datum_value> shared_root_datum;

struct inner_class_value : public value_base
{ complexredgp::ComplexReductiveGroup& val;
  complexredgp::ComplexReductiveGroup& dual;
  size_t& ref_count;

  lietype::Layout type_info;
  const realform_io::Interface interface,dual_interface;

  inner_class_value(std::auto_ptr<complexredgp::ComplexReductiveGroup> G
   , const lietype::Layout& lo); // main constructor
  ~inner_class_value();

  virtual void print(std::ostream& out) const;
  inner_class_value* clone() const 
    { return new inner_class_value(*this); }
  static const char* name() { return "inner class"; }

  inner_class_value(const inner_class_value& v); // copy constructor
  inner_class_value(const inner_class_value& v,tags::DualTag);
   // constructor of dual
};

typedef std::auto_ptr<inner_class_value> inner_class_ptr;
typedef std::tr1::shared_ptr<inner_class_value> shared_inner_class;

struct real_form_value : public value_base
{ const inner_class_value parent;
  realredgp::RealReductiveGroup val;

  real_form_value(inner_class_value p,realform::RealForm f)
  : parent(p), val(p.val,f) {}
  ~real_form_value() {} // everything is handled by destructor of |parent|

  virtual void print(std::ostream& out) const;
  real_form_value* clone() const 
    { return new real_form_value(*this); }
  static const char* name() { return "real form"; }
protected:
  real_form_value(inner_class_value,realform::RealForm,tags::DualTag);
     // for dual forms
  real_form_value(const real_form_value& v)
  : parent(v.parent), val(v.val) {} // copy c'tor
};

typedef std::auto_ptr<real_form_value> real_form_ptr;
typedef std::tr1::shared_ptr<real_form_value> shared_real_form;

struct dual_real_form_value : public real_form_value
{ dual_real_form_value(inner_class_value p,realform::RealForm f)
  : real_form_value(p,f,tags::DualTag()) {}

  virtual void print(std::ostream& out) const;
  dual_real_form_value* clone() const 
    { return new dual_real_form_value(*this); }
  static const char* name() { return "dual real form"; }
private:
  dual_real_form_value(const dual_real_form_value& v)
  : real_form_value(v)  {}
};

typedef std::auto_ptr<dual_real_form_value> dual_real_form_ptr;
typedef std::tr1::shared_ptr<dual_real_form_value> shared_dual_real_form;

struct Cartan_class_value : public value_base
{ const inner_class_value parent;
  size_t number;
  const cartanclass::CartanClass& val;

  Cartan_class_value(const inner_class_value& p,size_t cn);
  ~Cartan_class_value() {} // everything is handled by destructor of |parent|

  virtual void print(std::ostream& out) const;
  Cartan_class_value* clone() const 
    { return new Cartan_class_value(*this); }
  static const char* name() { return "Cartan class"; }
private:
  Cartan_class_value(const Cartan_class_value& v)
  : parent(v.parent), number(v.number), val(v.val) {}
};

typedef std::auto_ptr<Cartan_class_value> Cartan_class_ptr;
typedef std::tr1::shared_ptr<Cartan_class_value> shared_Cartan_class;


void initialise_builtin_types();

} }
#endif

