
#ifndef PARSETREE_H
#define PARSETREE_H
#if __cplusplus
#include <iostream>
extern "C" {
#endif


typedef short id_type;

typedef struct exprlist_node* expr_list;

typedef struct application_node* app;

typedef struct pattern_node* patlist;
struct id_pat
{ patlist sublist;
  id_type name;
  unsigned char kind; /* bit 0: has name, bit 1: has sublist */
};
struct pattern_node
{ patlist next;
  struct id_pat body;
};

typedef struct let_expr_node* let;

typedef void* ptr;

typedef struct lambda_node* lambda;

typedef struct conditional_node* cond;

typedef struct while_node* w_loop;
typedef struct for_node* f_loop;
typedef struct cfor_node* c_loop;

typedef struct subscription_node* sub;

typedef struct cast_node* cast;

typedef struct op_cast_node* op_cast;

typedef struct assignment_node* assignment;

typedef struct comp_assignment_node* comp_assignment;

typedef struct sequence_node* sequence;

union expru { 
              int int_denotation_variant;
              char* str_denotation_variant;
              
              id_type identifier_variant;
              
              expr_list sublist;
              
              app call_variant;
              
              let let_variant;
              
              lambda lambda_variant;
              
              cond if_variant;
              
              w_loop while_variant;
              f_loop for_variant;
              c_loop cfor_variant;
              
              sub subscription_variant;
              
              cast cast_variant;
              
              op_cast op_cast_variant;
              
              assignment assign_variant;
              
              comp_assignment comp_assign_variant;
              
              sequence sequence_variant;
 };

typedef enum 
{ 
  integer_denotation, string_denotation, boolean_denotation, 
  
  applied_identifier, 
  
   list_display, 
  
   tuple_display, 
  
   function_call, 
  
   let_expr, 
  
  lambda_expr,
  
   conditional_expr, 
  
   while_expr, for_expr, cfor_expr, 
  
   subscription, 
  
  cast_expr, 
  
  op_cast_expr, 
  
  ass_stat, 
  
  comp_ass_stat, 
  
  seq_expr, 
  } expr_kind;
typedef struct { union expru e; expr_kind kind; } expr;


struct exprlist_node { expr e; expr_list next; };

struct application_node { expr fun; expr arg; };

typedef struct partial_formula* form_stack;
struct partial_formula
{ expr left_subtree; id_type op; int prio;
  form_stack prev;
};

typedef struct let_node* let_list;
struct let_node { struct id_pat pattern; expr val; let_list next; };
struct let_expr_node { struct id_pat pattern; expr val; expr body; };

struct lambda_node
{ struct id_pat pattern; ptr arg_type; expr body; };

struct conditional_node
 { expr condition; expr then_branch; expr else_branch; };

struct while_node
 { expr condition; expr body; expr next_part; };
struct for_node
 { struct id_pat id; expr in_part; expr body; };
struct cfor_node
 { id_type id; expr count; expr bound; short up; expr body; };

struct subscription_node { expr array; expr index; };

struct cast_node { ptr type; expr exp; };

struct op_cast_node { id_type oper; ptr type; };

struct assignment_node { id_type lhs; expr rhs; };

struct comp_assignment_node { id_type aggr; expr index; expr rhs; };

struct sequence_node { expr first; expr last; };



void destroy_expr(expr e);

expr make_int_denotation (int val);
expr make_string_denotation(char* val);
expr make_bool_denotation(int val);

expr make_applied_identifier (id_type id);

void destroy_exprlist(expr_list l);

extern const expr_list null_expr_list;
expr_list make_exprlist_node(expr e, expr_list l);
expr_list reverse_expr_list(expr_list l);
expr wrap_list_display(expr_list l);

expr wrap_tuple_display(expr_list l);

expr make_application_node(expr f, expr_list args);

expr make_unary_call(id_type name, expr arg);
expr make_binary_call(id_type name, expr x, expr y);

void destroy_letlist(let_list l);

let_list add_let_node(let_list prev, struct id_pat pattern, expr val);
expr make_let_expr_node(let_list decls, expr body);

expr make_lambda_node(patlist patl, ptr typel, expr body);

expr make_conditional_node(expr c, expr t, expr e);

expr make_while_node(expr c, expr b, expr n);
expr make_for_node(struct id_pat id, expr ip, expr b);
expr make_cfor_node(id_type id, expr count, expr bound, short up, expr b);

expr make_subscription_node(expr a, expr i);

expr make_cast(ptr type, expr exp);

expr make_op_cast(id_type name,ptr type);

expr make_assignment(id_type lhs, expr rhs);

expr make_comp_ass(expr lhs, expr rhs);

expr make_sequence(expr first, expr last);

short lookup_identifier(const char*);
void include_file();

void global_set_identifier(struct id_pat id, expr e, int overload);
void global_declare_identifier(id_type id, ptr type);
void show_ids();
void type_of_expr(expr e);
void show_overloads(id_type id);


extern struct partial_formula dummy_formula;
const form_stack unary_marker=&dummy_formula;

form_stack start_formula (expr e, id_type op, int prio);
form_stack start_unary_formula (id_type op, int prio);
form_stack extend_formula (form_stack pre, expr e,id_type op, int prio);
expr end_formula (form_stack pre, expr e);
void destroy_formula(form_stack s);

patlist make_pattern_node(patlist next,struct id_pat* body);
void destroy_pattern(patlist p);
void destroy_id_pat(struct id_pat* p);
patlist reverse_patlist(patlist p);

ptr mk_type_singleton(ptr t);
ptr mk_type_list(ptr t,ptr l);
ptr mk_prim_type(int p);
ptr mk_row_type(ptr c);
ptr mk_tuple_type(ptr l);
ptr mk_function_type(ptr a,ptr r);

void destroy_type(ptr t);
void destroy_type_list(ptr t);

#if __cplusplus
}

namespace atlas
{ namespace interpreter
  {

std::ostream& operator<< (std::ostream& out, expr e);

size_t length(expr_list l);

bool is_empty(const expr& e);

std::ostream& operator<< (std::ostream& out, const id_pat& p);

ptr first_type(ptr typel);
std::ostream& print_type(std::ostream& out, ptr type);

  }
}

#endif
#endif

